@@ -1,5 +1,6 @@
 import functools
 import os
+import re
 import sys
 import threading
 import warnings
@@ -10,7 +11,7 @@ import pytest
 
 from pytest_run_parallel.thread_comparator import ThreadComparator
 from pytest_run_parallel.thread_unsafe_detection import (
-    THREAD_UNSAFE_FIXTURES,
+    construct_thread_unsafe_fixtures,
     identify_thread_unsafe_nodes,
 )
 from pytest_run_parallel.utils import (
@@ -19,6 +20,18 @@ from pytest_run_parallel.utils import (
     get_num_workers,
 )
 
+GIL_WARNING_MESSAGE_CONTENT = re.compile(
+    r"The global interpreter lock \(GIL\) has been enabled to load module '(?P<module>[^']*)'"
+)
+
+GIL_ENABLED_ERROR_TEXT = (
+    "GIL was dynamically re-enabled during test {stage_test} to load module '{module}'. "
+    "When running under a free-threaded interpreter with the GIL initially disabled, "
+    "the test suite must not cause the GIL to be re-enabled at runtime. Check "
+    "for compiled extension modules that do not use the 'Py_mod_gil' slot or the "
+    "'PyUnstable_Module_SetGIL' API."
+)
+
 
 def wrap_function_parallel(fn, n_workers, n_iterations):
     @functools.wraps(fn)
@@ -92,6 +105,10 @@ class RunParallelPlugin:
     def __init__(self, config):
         self.verbose = bool(int(os.environ.get("PYTEST_RUN_PARALLEL_VERBOSE", "0")))
         self.skip_thread_unsafe = config.option.skip_thread_unsafe
+        self.mark_warnings_as_unsafe = config.option.mark_warnings_as_unsafe
+        self.mark_ctypes_as_unsafe = config.option.mark_ctypes_as_unsafe
+        self.mark_hypothesis_as_unsafe = config.option.mark_hypothesis_as_unsafe
+        self.ignore_gil_enabled = config.option.ignore_gil_enabled
 
         skipped_functions = [
             x.split(".") for x in config.getini("thread_unsafe_functions")
@@ -100,10 +117,7 @@ class RunParallelPlugin:
             (".".join(x[:-1]), x[-1]) for x in skipped_functions
         )
 
-        self.unsafe_fixtures = THREAD_UNSAFE_FIXTURES | set(
-            config.getini("thread_unsafe_fixtures")
-        )
-
+        self.unsafe_fixtures = construct_thread_unsafe_fixtures(config)
         self.thread_unsafe = {}
         self.run_in_parallel = {}
 
@@ -136,7 +150,13 @@ class RunParallelPlugin:
             used_unsafe_fixtures = self.unsafe_fixtures & set(fixtures)
             return True, f"uses thread-unsafe fixture(s): {used_unsafe_fixtures}"
 
-        return identify_thread_unsafe_nodes(item.obj, self.skipped_functions)
+        return identify_thread_unsafe_nodes(
+            item.obj,
+            self.skipped_functions,
+            self.mark_warnings_as_unsafe,
+            self.mark_ctypes_as_unsafe,
+            self.mark_hypothesis_as_unsafe,
+        )
 
     @pytest.hookimpl(trylast=True)
     def pytest_itemcollected(self, item):
@@ -258,6 +278,30 @@ class RunParallelPlugin:
         else:
             terminalreporter.line("All tests were run in parallel! 🎉")
 
+    @pytest.hookimpl(tryfirst=True)
+    def pytest_warning_recorded(
+        self, warning_message: warnings.WarningMessage, when, nodeid, location
+    ):
+        mo = re.match(GIL_WARNING_MESSAGE_CONTENT, str(warning_message.message))
+        if mo is None or self.ignore_gil_enabled:
+            return
+
+        if when == "collect":
+            stage = "collection"
+        elif when == "runtest":
+            stage = "execution"
+        else:
+            stage = "configuration"
+        stage_test = stage
+        if nodeid:
+            stage_test += f" of '{nodeid}'"
+        pytest.exit(
+            reason=GIL_ENABLED_ERROR_TEXT.format(
+                stage_test=stage_test, module=mo.group("module")
+            ),
+            returncode=1,
+        )
+
 
 @pytest.fixture
 def num_parallel_threads(request):
@@ -316,6 +360,30 @@ def pytest_addoption(parser):
         type=bool,
         default=False,
     )
+    parser.addoption(
+        "--mark-warnings-as-unsafe",
+        action="store_true",
+        dest="mark_warnings_as_unsafe",
+        default=False,
+    )
+    parser.addoption(
+        "--mark-ctypes-as-unsafe",
+        action="store_true",
+        dest="mark_ctypes_as_unsafe",
+        default=False,
+    )
+    parser.addoption(
+        "--mark-hypothesis-as-unsafe",
+        action="store_true",
+        dest="mark_hypothesis_as_unsafe",
+        default=False,
+    )
+    parser.addoption(
+        "--ignore-gil-enabled",
+        action="store_true",
+        dest="ignore_gil_enabled",
+        default=False,
+    )
     parser.addini(
         "thread_unsafe_fixtures",
         "list of thread-unsafe fixture names that cause a test to "
